
#include "mc/manager-mc.h"
#include "mc/reply-mc.h"
#include "mc/common-mc.h"

/* 
 * Fetch the next reply for this key from the outstanding replies
 * structure. If all the replies have been cancelled since this
 * reply was scheduled, the return value will be -1. Otherwise,
 * the list will contain an offset and a length to send. 
 */

/* 
 * Reply format:
 * 32-bit offset
 * 32-bit page length
 * 16-bit keylen
 * key
 * data
 */
// virtual 
int 
MC_Reply::fill_ADU(void) 
{
	/* forward the pointer; should be done in srm lib but isn't*/
	char *pkt = (char *)(pb_->data+sizeof(srmhdr));

	dprintf(stderr, "REPLY::FILL_ADU():%s ", key_);

	/* need to get holes from tcl to us */
	int argc; 
	char **argv;
	Tcl& tcl = Tcl::instance();	
	tcl.evalf("%s get_oreply %s", mcmgr_->name(), key_);		
	Tcl_SplitList(tcl.interp(), tcl.result(), &argc, &argv);
	int offset = atoi(argv[0]);
	dprintf(stderr, " offset:%d", offset);
	if (offset == -1) {
		/* all replies on this url cancelled */
		dprintf(stderr, " ALREADY CANCELLED\n");
		this->cancel();
		free(argv);
		return 0;
	}
	int len = atoi(argv[1]);
	dprintf(stderr, " len:%d\n", len);

	CacheBuf* cb = mcmgr_->lookup_by_key(key_);
	if (cb == 0) {
		/* we no longer have the data */
		/* XXX need to remove the oreplies */
		dprintf(stderr, " DATA GONE\n");
		this->cancel();
		free(argv);
		return 0;
	}

	int keylen = strlen(key_);
	int nbytes = len; // remember len
	if (nbytes > SRM_MTU - keylen) {
		nbytes = SRM_MTU - keylen;
	}
	int hlen = keylen + 10;

	*(u_int32_t*)&pkt[0] = htonl(offset);
	*(u_int32_t*)&pkt[4] = htonl(cb->length());
	*(u_int16_t*)&pkt[8] = htons(keylen);	
	memcpy((char*)&pkt[10], key_, keylen);
	memcpy((char *)&pkt[hlen], cb->data() + offset, nbytes);

//	tcl.evalf("mc_trace_rp %d %d %s", offset, nbytes, key_);

	dprintf(stderr, " OFFSET:%d NBYTES:%d\n", offset, nbytes);

	int leftover = len - nbytes;
	if (leftover > 0) {
		tcl.evalf("%s add_reply %s %d %d", 
			  mcmgr_->name(), key_, offset+nbytes, leftover);
	}

	free(argv);
	return nbytes + hlen;
}
