#ifndef MASH_MPLUG_H
#define MASH_MPLUG_H

#include "cc-api/np-plugin.h"
#include "rpi.h"
#include "mplug-config.h"


class MPlugInstance : public PluginInstance {
public:
	MPlugInstance(NPMIMEType mimeType, NPP instance, uint16 mode,
		      int16 argc, const char *const *argn,
		      const char *const *argv);
	virtual ~MPlugInstance();

	long SendData(int32 offset, int32 len, void *buffer);
	NPError StreamDone(NPError reason);

protected:
	uint16 mode() { return mode_; }
	NPBool NotifyReconfigure(uint32 width, uint32 height);
	
	virtual NPError NewWindow(NPWindow *npWindow);
	virtual NPError ReconfigureWindow(NPWindow *npWindow);
	virtual NPError DeleteWindow() = 0;
	virtual const char *GetDisplay(NPWindow *npWindow) = 0;
	virtual unsigned long GetWindowHandle() = 0;

private:
	PluginStream* CreateStream(NPMIMEType type, NPStream *stream, 
				   NPBool seekable, uint16 *stype);

	NPError SetWindow(NPWindow* window);
	NPBool  NewProcess(NPWindow *npWindow);
	NPBool  NotifyStreamData(int32 len, void *buffer);
	NPBool  NotifyURL(const char *url);
	
	RPI rpi_;
	char *mimeType_;
	uint16 mode_;
	int16 argc_;
	char **argn_;
	char **argv_;
	char *url_;
	char *data_;
	int32 dataLen_;  /* negative dataLen_ implies I've finished receiving
			  * all the data */
	
	static MPlugConfiguration *GetConfig() {
		if (!config_) config_ = new MPlugConfiguration();
		return config_;
	}
	
	static MPlugConfiguration *config_;
	friend class PluginInstance;
};


class MPlugStream : public PluginStream {
public:
	MPlugStream(MPlugInstance *plugin, NPStream *stream)
		: PluginStream(stream), plugin_(plugin) { }
	~MPlugStream() { }

	virtual long Write(int32 offset, int32 len, void *buffer)
	{ return plugin_->SendData(offset, len, buffer); }
	virtual NPError Destroy(NPError reason)
	{ return plugin_->StreamDone(reason); }
	
private:
	MPlugInstance *plugin_;
};



inline PluginStream*
MPlugInstance::CreateStream(NPMIMEType /*type*/, NPStream *stream, 
			    NPBool /*seekable*/, uint16 * /*stype*/)
{
	MPlugStream *mplugStream = new MPlugStream(this, stream);
	if (mplugStream) {
		NotifyURL(stream->url);
	}
	return mplugStream;
}



#if defined(XP_UNIX)

#include "unix/mplug-unix.h"
#define MPLUG_INSTANCE_CLASS UnixMPlugInstance

#elif defined(XP_WIN)

#include "win32/mplug-win32.h"
#define MPLUG_INSTANCE_CLASS Win32MPlugInstance

#endif


#endif /* MASH_MPLUG_H */

