#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: director.tcl
#   -- Created on Wed Aug 12 1998
#   -- Author: Yatin Chawathe <yatin@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/mashcast/director.tcl,v 1.3 1998/09/03 02:37:25 yatin Exp $
#


import AnnounceListenManager Timer/Adaptive/ConstBW


Class CastingDirector_v2 -superclass AnnounceListenManager


CastingDirector_v2 public init { {addr 224.8.8.8/6666/none/16} } {
	$self next $addr
	$self set callback_ {}
	$self set latest_timestamp_ 0

	set timer [new Timer/Adaptive/ConstBW 1000]
	$timer randomize
	$timer update_nsrcs 1
	$self set nsrcs_ 1
	$self set url_ ""
	$self timer $timer
}


CastingDirector_v2 public cast { url } {
	puts "%%%%%%%%%%%%% starting to cast"
	$self instvar latest_timestamp_ url_ callback_
	set url_ $url
	if { $callback_!={} } { $callback_ geturl $url }
	set latest_timestamp_ [clock clicks]
	$self set_announcement [list $url $latest_timestamp_]
	$self start
	puts "%%%%%%%%%%%%% end of cast"
}


CastingDirector_v2 public callback { c } {
	$self set callback_ $c
}


#
# The data format is
#    $url $timestamp
#
CastingDirector_v2 private recv_announcement { srcaddr srcport data len } {
	$self instvar latest_timestamp_ url_ callback_ sources_ nsrcs_

	puts "received $data from $source"
	if ![info exists sources_($srcaddr:$srcport)] {
		# update the adaptive timer
		set sources_($srcaddr:$srcport) 1
		incr nsrcs_
		[$self get_timer] update_nsrcs $nsrcs_
	}
	
	set url [lindex $data 0]
	set ts  [lindex $data 1]
	if { $ts > $latest_timestamp_ } {
		# we got a newer announcement
		$self stop

		if { $url!=$url_ && $callback_!={} } {
			$callback_ geturl $url
		}
		set url_ $url
		set latest_timestamp_ $ts
	}
}

