#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: remote-controller.tcl
#   -- Created on Thu Aug 13 1998
#   -- Author: Yatin Chawathe <yatin@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/mashcast/remote-controller.tcl,v 1.2 1998/08/18 22:54:52 yatin Exp $
#


import TCP


Class RemoteController -configuration {
	remoteControllerPort 3470
}


RemoteController public init { {munge_url_port {}} } {
	$self next
	$self set munge_url_port_ $munge_url_port
	$self set my_port_ {}
}


RemoteController public destroy { } {
	$self instvar server_ conn_
	if [info exists conn_] { delete $conn_ }
	if [info exists server_] { delete $server_ }
}


RemoteController public use_socket { {my_port {}} {callback {}} } {
	$self set callback_ $callback
	$self set my_port_ $my_port

	if { $my_port=={} } {
		set my_port [$self get_option remoteControllerPort]
	}
	
	$self instvar server_
	set server_ [new TCP/Server]
	$server_ open $my_port "$self create_channel"
}


RemoteController private create_channel { chan } {
	$self instvar conn_ callback_
	set conn_ [new TCP]
	$conn_ open $chan
	if { $callback_!={} } { eval $callback_ }
}


RemoteController public geturl { url } {
	$self instvar munge_url_port_ my_port_ conn_

	if { $munge_url_port_!={} } {
		set url "http://127.0.0.1:${munge_url_port_}/$url"
	}
	if { $my_port_!={} } {
		if [info exists conn_] {
			$conn_ sendline "show: $url"
		}
	} else {
		# use "netscape -remote"
		# will work only on UNIX
		catch {exec netscape -remote openURL(${url})} msg
		if { $msg!={} } {
			puts stderr $msg
		}
	}
}
