#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: ui-caster.tcl
#   -- Created on Wed Aug 12 1998
#   -- Author: Yatin Chawathe <yatin@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/mashcast/ui-caster.tcl,v 1.1 1998/08/14 22:19:52 yatin Exp $
#



import WidgetClass
WidgetClass transparent_gif


import DropDown/Text


WidgetClass CastingDirectorUI -default {
	{ .urllist.highlightThickness 1 }
	{ *Checkbutton.borderWidth 1 }
	{ *Checkbutton.font WidgetDefault }
	{ *Checkbutton.activeBackground WidgetDefault(-background) }
	{ *Button.borderWidth 1 }
	{ *Menubutton.borderWidth 1 }
	{ *Menubutton*font WidgetDefault }
	{ *Entry.borderWidth 1 }
	{ *Entry.width 40 }
	{ *Entry.background white }
	{ *Entry.foreground black }
}


CastingDirectorUI public add_url { url } {
	$self subwidget urllist insert end $url
}


CastingDirectorUI public director { d } {
	$self set director_ $d
}


CastingDirectorUI private build_widget { path } {
	$self tkvar url_ cast_to_everyone_

	# set url_ to a non-empty string to prevent the trace
	# mechanism from being invoked when you insert items into
	# the dropdown widget
	set url_ " "
	
	frame $path.frame
	frame $path.buttons
	
	label $path.label -text "Enter or select URL:" -justify left -anchor w
	DropDown/Text $path.urllist -variable [$self tkvarname url_]
	checkbutton $path.castmode -text "Cast to everyone" -anchor w \
			-variable [$self tkvarname cast_to_everyone_]
	set cast_to_everyone_ 1
	button $path.quit -text "Quit" -command "exit" -pady 1
	
	pack $path.label   -in $path.frame -side left -fill y
	pack $path.urllist -in $path.frame -side left -fill both -expand 1
	
	pack $path.castmode -in $path.buttons -side left -padx 15 -fill x \
			-expand 1
	pack $path.quit -in $path.buttons -side right -padx 5
	
	pack $path.frame -side top -fill x
	pack $path.buttons -side bottom -fill both -expand 1
	focus [$path.urllist subwidget entry]

	trace variable url_ w "$self cast"
}


CastingDirectorUI private cast { args } {
	$self tkvar url_
	set url_ [string trim $url_]
	if { $url_=={} } return
	
	$self instvar director_
	if [info exists director_] {
		$director_ cast $url_
	}
}


CastingDirectorUI public geturl { url } {
	puts "director asked us to fetch $url"
}


CastingDirectorUI public enable { flag } {
	set path [$self info path]
	if { $flag } {
		set state normal
		set fg [$path.castmode cget -foreground]
	} else {
		set state disabled
		set fg [$path.castmode cget -disabledforeground]
	}
	$path.castmode configure -state $state
	$path.urllist configure -state $state
	$path.label configure -foreground $fg
}
