#
# Copyright (c) 1997 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Header: /usr/src/mash/repository/mash/mash-1/tcl/mc/launcher.tcl,v 1.11 1998/06/28 17:30:13 kwright Exp $
#
#XXX
source ../tcl8.0/library/http2.0/http.tcl
#source http.tcl
# to autoload the ::http namespace
::http::formatQuery sdsds

Class Launcher

Launcher public init {webcache} {
	$self instvar webcache_ 
	set webcache_ $webcache
}

Launcher private progress {token total current} {
#	puts stderr "$token $current OF $total"
	flush stderr
}

# Checks to see if we have a waiting proxy connection.
# If so, uploads the data. Calling method has put the 
# data in buffer$webcache_
Launcher instproc upload {url len type } {
	$self instvar pc_
#puts "LAUNCHER UPLOAD: $url $self"
	if {[$self check_for_outstanding_url $url] != -1} {
                # unset the proxy connection because we don't want to 
                # keep uploading in a loop.
                set pc $pc_($url)
                unset pc_($url)
		$pc upload $len $type
		mc_trace_ul $url $len
	}
}

Launcher private page_ready {token} {
	$self instvar pc_ url_
	set url $url_($token)
	flush stderr
	#XXX check against token_
	upvar #0 $token state

	if [$self check_for_redirect $token] {
		return
	}

	set status [::http::status $token]
	if { $status != "ok" } {
		puts stderr "PAGE_READY ERROR $code"
	} else {
		# forward the reply to multicast participants
		set len [::http::size $token]
		set type [$self content_type $token]
		#
		# Stuff the URL into the srm webcache session.
		#
	#XXX doesn't work to pass data as an arg because OTclDispatch
	# sets up a string-based argv... the right solution is to fix
	# otcl, but for now, we just pass the buffer name in a global
		$self instvar webcache_
		global buffer$webcache_
		set buffer$webcache_ [::http::data $token]
		$webcache_ insert $url $len $type
#		puts stderr "INSERT $url $len ($type) "
	}
}

Launcher instproc update_proxy_connection {url proxyconnection} {
	$self instvar pc_
#	puts "UPDATE_PC: $url $proxyconnection"
        if {$proxyconnection != 0} {
		set pc_($url) $proxyconnection
	}
}

Launcher instproc launch {url proxyconnection} {
	#XXX should use -channel option to splice data
	#directory into webcache, but we don't have C
	#hooks for this yet (need to add a channel type?)
	$self instvar token_ url_ pc_ 
	$self update_proxy_connection $url $proxyconnection

	set token_ [::http::geturl $url -command "$self page_ready" \
			-progress "$self progress"]
        # remember the url we're fetching
        set url_($token_) $url
}



# Check if this url is outstanding. If so,
# return 1; if not outstanding, return -1.
Launcher instproc check_for_outstanding_url url {
	$self instvar pc_
	set urls [array names pc_] 
	if {[lsearch $urls $url] != -1} {
		return 1
	} else {
		return -1
	}
}

Launcher private check_for_redirect token {
	upvar #0 $token state
	set max 0
	flush stderr
	foreach {name value} $state(meta) {
                if {[string length $name] > $max} {
			set max [string length $name]
                }
                if {[regexp -nocase ^location$ $name]} {
#			puts stderr "Location:$value"
			$self instvar url_ pc_ webcache_
			set oldurl $url_($token)
			set newurl [string trim $value]

			$webcache_ update_url $oldurl $newurl 

                        set pc 0
			if {[$self check_for_outstanding_url $oldurl] != -1} {
				set pc $pc_($oldurl)
                                unset pc_($oldurl)
			} 

			$self launch $newurl $pc
			return 1
                }
	}
	incr max
#	foreach {name value} $state(meta) {
#                puts [format "%-*s %s" $max $name: $value]
#	}
	return 0
}


Launcher private content_type token {
	upvar #0 $token state
	foreach {name value} $state(meta) {
                if {[regexp -nocase ^content-type$ $name]} {
			return [string trim $value]
                }
	}
	return ""
}

