# This file contains a simple graph comm module to be
# used for graph interface communication. 

import AnnounceListenManager

Class AnnounceListenManager/GraphComm -superclass AnnounceListenManager

AnnounceListenManager/GraphComm instproc init {mtu spec} {
    $self next $spec $mtu;
    
    $self instvar next_mesg_id_;
    $self instvar local_callback_;
    $self instvar local_id_;
    
    set next_mesg_id_ 0;
    set local_callback_ "";
    set local_id_ "";
}

AnnounceListenManager/GraphComm instproc register {srcid cb} {
    $self instvar local_callback_;
    $self instvar local_id_;
    
    set local_id_ $srcid;
    set local_callback_ $cb;
}

AnnounceListenManager/GraphComm instproc recv_announcement {oaddr port mesg length} {
    
    # The first cut of this has the following mesg format:
    # Src MesgId RefId Mesg

    $self instvar local_id_;

    set src [lindex $mesg 0];
    set mid [lindex $mesg 1];
    set rid [lindex $mesg 2];
    set deliver_once_flag [lindex $mesg 4];
    set mesg [lindex $mesg 3];

    $self instvar local_callback_

    global kpatel_debug;

    if {$deliver_once_flag} {
	$self instvar delivery_record;
	
	if {[info exists delivery_record($src,$mid)]} {
	    if {[info exists kpatel_debug]} {
		$kpatel_debug log_graph_comm 0 0 $mesg;
	    }
	    return
	}
	set delivery_record($src,$mid) 1;
    }

    if {[info exists kpatel_debug]} {
	$kpatel_debug log_graph_comm 0 1 $mesg;
    }

    if {$local_callback_ != ""} {
	after 1 $local_callback_ [list $rid $mesg];
    }
    
    return;
}

AnnounceListenManager/GraphComm instproc do_announce {src rid mesg {repeat_period 0} {deliver_once_flag 0}} {
    $self instvar next_mesg_id_;

    incr next_mesg_id_;
    set mid $next_mesg_id_;
    
    if {$rid == "-1"} {
	set rid "${src}:${mid}";
    }

    set wire_mesg [list $src $mid $rid $mesg $deliver_once_flag];

    global kpatel_debug;

    if {[info exists kpatel_debug]} {
	$kpatel_debug log_graph_comm 1 0 $wire_mesg;
    }
    
    $self announce $wire_mesg;
    
    if {$repeat_period != 0} {
	$self instvar resend_array;
	
	set resend_array(${src}:${mid},period) $repeat_period;
	set resend_array(${src}:${mid},mesg) $wire_mesg;
	after $repeat_period $self resend ${src}:${mid};
    }
    return "${src}:${mid}";
}

AnnounceListenManager/GraphComm instproc resend {mid} {
    $self instvar resend_array;

    if {![info exists resend_array($mid,period)]} {
	return;
    }

    if {$resend_array($mid,period) == 0} {
	unset resend_array($mid,period);
	unset resend_array($mid,mesg);
	return;
    }

    global kpatel_debug;

    if {[info exists kpatel_debug]} {
	$kpatel_debug log_graph_comm 1 0 $resend_array($mid,mesg);
    }

    $self announce $resend_array($mid,mesg);

    after $resend_array($mid,period) $self resend $mid;
}

AnnounceListenManager/GraphComm instproc stop_resend {mid} {
    $self instvar resend_array;

    if {![info exists resend_array($mid,period)]} {
	return;
    }
    
    set resend_array($mid,period) 0;
}


