#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: ssac.tcl
#   -- Created on Mon Mar  2 1998
#   -- Author: Angela Marie Schuett <schuett@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/ssac/client-ssac.tcl,v 1.15 1998/09/26 22:22:37 schuett Exp $
#

import mashutils SDPParser AnnounceListenManager/AS/Client/SSAC SessionCatalog SDPParser

# Client side SSAC parsing
# Status: pre-alpha
# Author: Angela Schuett

Class SSAC_Client


SSAC_Client public init {} {
	$self instvar first_

	set first_ 0
}

SSAC_Client public init_vars {app} {
	$self instvar rtsp_ session_
	
	set rtsp_(state) IDLE
	set rtsp_(application) $app
	set session_(uri_) ""
	set session_(seqno) 1
	

}


SSAC_Client public send {method} {
	$self instvar url_ session_ AS_ first_ rtsp_
	
	if {$first_ == 0} {
	    
		set AS_ [new AnnounceListenManager/AS/Client/SSAC $self $session_(server_name_) 20000 static:mars $session_(rport)]
	}
	switch -exact -- $method {
		"PLAY" {

			$AS_ set_offset "Offset [$rtsp_(application) offset]"
			$AS_ set_seqno "Seqno [incr session_(seqno) 2]"
		        $AS_ set_filename $session_(session_name_)
			
			
			
		}
		"PAUSE" {
			$AS_ set_offset "Offset PAUSE"
			$AS_ set_seqno "Seqno [incr session_(seqno)]"
		}
			
		"CLOSE" {
			# XXX FIX THIS

			#$AS_ set_offset "Offset End"
			#$AS_ set_seqno "Seqno [incr session_(seqno)]"
			
		}
	}

	#send the message

	if {$first_ == 0} {
		
		$AS_ start 
		set first_ 1
	} else {
		#$AS_ change_state $msg
		$AS_ send_announcement
		# Seek, pause, close are sent immediately, without waiting for timer to fire.  Keep-alive msgs are sent on timer
	}
	
}

SSAC_Client public notify {msg} {
	
	#puts "Notify: $msg"
	# set timescales

	$self parse_server_msg $msg

	
	
	
}

SSAC_Client public open {url desc} {
	$self instvar session_ rtsp_ sesslist_ 

	set session_(uri_) $url

	$self parse_pd $desc
	
	
	# for now, we assume that the start time is now + 15 seconds
	#set session_(startwall) [expr [clock seconds] + 15]
	#puts "OPEN $session_(startwall)"
	$self send "PLAY"
	
	
	return ""

}

SSAC_Client public playOrPause { {stop ""}} {
	$self instvar rtsp_
	
	if {$stop == "STOP"} {
		set rtsp_(state) "PLAY"
	}
	if {$rtsp_(state) == "PLAY"}  {
		$self pause
	} else {
		$self play
	}

}

SSAC_Client public duration { } {
	$self instvar session_
	return [expr $session_(endtime_) - $session_(starttime_)]
}


SSAC_Client public elapsed { time } {
	$self instvar session_
	return [duration_readable $time]
}


# This procedure parses the server SSAC announcement
SSAC_Client private parse_server_msg {msg} {
	$self instvar session_ sesslist_ first_
	$self instvar rtsp_  AS_ old_offset_

	
	set o [split $msg "\n"]

	set seqno [lindex [split [lindex $o 0]] 1]
	if {$seqno >= $session_(seqno)} {
		set session_(seqno) $seqno
		#set session_(STS) [lindex [split [lindex $o 2]] 1]
	
		set duration [lindex [split [lindex $o 2]] 1]
		#puts "duration: $duration"
		
		set session_(starttime_) 0
		set session_(endtime_) $duration
		
		set systemt [lindex [split [lindex $o 3]] 1]
		set offsett [lindex [split [lindex $o 3]] 2]
		#puts "system time=$systemt  logical time=$offsett"
		set system_offset [expr [clock seconds] - $systemt]
		set new_offset [expr $offsett + $system_offset]
		
		set len [llength $o]
		for {set n 4} {$n < $len} {incr n 2} {
			set sess [lindex [split [lindex $o $n]] 1]
			if {![lsearch sesslist_ $sess]} {
				puts  "Error"
			}
			##
			set tmpsession($sess) [lindex [split [lindex $o [expr $n + 1]]] 1]
			
			
		}

		
		if {$first_ == 1} {
			foreach sess $sesslist_ {
				set session_($sess,Address) $tmpsession($sess)
			}
			$rtsp_(application) activate_slider
			set rtsp_(state) "PLAY"
		
			set session_(duration_) [expr $session_(endtime_)-$session_(starttime_)]
			$rtsp_(application) update-title $session_(session_info1_) $session_(session_info2_) $session_(duration_)
			$rtsp_(application) play-update "pause"
		
			$self start_tools
		
		
			set first_ 2
		} else {
		
			set addrchange 0
			# Fix this
			$rtsp_(application) set-offset $new_offset
			$AS_ set_offset "Offset $new_offset"
			
			foreach sess $sesslist_ {
				if {$tmpsession($sess) != $session_($sess,Address)} {
				        set addrchange 1
					set session_($sess,Address) $tmpsession($sess)
					
				}
			}
			if {$addrchange == 1} {
				$self reset_tools
				set addrchange 0
			}
		
		}
		#$AS_ set_timestamp "STS $session_(STS)"
		set old_offset_ $new_offset

		
		
	} else {
		#puts "OLD SEQNO $seqno $session_(seqno)"
		
	}
	
}




# This procedure parses the presentation description, which contains the
# server mcast address, catalog file location, and catalog file.  The pd is
# passed through the -server flag, and is intended to be downloaded from
# the web.  Shouldn't be in this class, move to application.

SSAC_Client private parse_pd {pd} {
	$self instvar session_ sesslist_ 
	
	

	set ctg [new SessionCatalog]
	$ctg parse $pd
	set sdp [$ctg get_sdp]
	#puts $sdp

	set p [new SDPParser]
	set message [$p parse $sdp]
   
    if {$message != ""} {
	if {[$message have_field s]} {
	    set session_(session_info1_) [$message field_value s]
	}
	if {[$message have_field i]} {
	 
		set session_(session_info2_) [$message field_value i]
	}
    } else {
	set session_(session_info1_) ""
	set session_(session_info2_) ""
    }
	set desc [$ctg get_desc]
    if {$desc==""} {
	error "Catalog file is missing server contact info"
    }

	set desclines [split $desc "\n"]
    
    # Instead of using -megactrl, should we pass in unicast server addr?
    if {[$self get_option megaVideoSession] != ""} {
	set session_(server_name_) [$self get_option megaVideoCtrl]
    } else {
	set server_name [lindex [split [lindex $desclines 1]] 1]
	regsub -all ":" $server_name "/" session_(server_name_)
    }
    set session_(rport) [$self get_option megaRecvVideoPort]
	set session_(session_name_) [lindex [split [lindex $desclines 0]] 1]
	
	
	
	foreach id [$ctg info streams] {
		lappend sessions([$ctg info session $id]) $id
	}
	set sesslist_ [array names sessions]
	#puts "sesslist= $sesslist_"


	foreach sess $sesslist_ {
		# XXX hack
		set session_($sess,Media) [string trimright $sess "0123456789"]
		#puts "********* $sess  $session_($sess,Media) **********"
		
	}
	
	# FIX THIS
	#set session_(video1,Media) "video"
	#set session_(audio1,Media) "audio"
	#set session_(mediaboard1,Media) "mediaboard"
	


}



SSAC_Client private play { } {
	$self instvar rtsp_ 
	if { $rtsp_(state) != "PLAY" } {
		set rtsp_(state) PLAY
		$rtsp_(application) play-update "Pause"
		$self send PLAY
	}
}      

SSAC_Client private pause { } {
	$self instvar rtsp_ 
	if { $rtsp_(state) != "PAUSE" } {
		set rtsp_(state) PAUSE
		$rtsp_(application) play-update "Play"
		$self send PAUSE
	}
}


# Called when the data addresses change, for example when the MARS
# server crashes and restarts.  Since mediaboard is currently started as
# an outside app, we'd have to just start a new one.
SSAC_Client private reset_tools {} {
	$self instvar session_ sesslist_ rtsp_

	foreach sess $sesslist_ {
		switch -- $session_($sess,Media) {
			video {
				
			    $rtsp_(application) reset_vic $session_($sess,Address)
				
	}
			audio {
				$rtsp_(application) reset_vat $session_($sess,Address)
			}
			mediaboard {
				puts "Not resetting Mediaboard**"
			}
	
		}
	}
}




SSAC_Client private start_tools {} {
	$self instvar session_ sesslist_ rtsp_
	foreach sess $sesslist_ {
		switch -- $session_($sess,Media) {
			video {
			#puts "Starting VIC..."
				$rtsp_(application) start_vic $session_($sess,Address)
			
			}
			audio {
				#puts "Starting VAT..."

				$rtsp_(application) start_vat $session_($sess,Address)
			}
			mediaboard {
			    global env
			    set mpid_ [eval exec mb -sa $session_($sess,Address) &]
			}
	
		}
	}
}

# Start the tools standalone
SSAC_Client private start_tools2 {} {
	$self instvar session_ sesslist_ rtsp_

	foreach sess $sesslist_ {
		switch -- $session_($sess,Media) {
			video {
				puts "Starting VIC $session_($sess,Address)"
				#$rtsp_(application) start_vic $session_($sess,Address)
				
				set vpid_ [eval exec vic -X loopback=1 $session_($sess,Address) &]

			}
			audio {
				puts "Starting VAT $session_($sess,Address)"
				#$rtsp_(application) start_vat $session_($sess,Address)
				set apid_ [eval exec vat -X loopback=1 $session_($sess,Address) &]
				
			}
			
			mediaboard {
				puts "Starting MB $session_($sess,Address)"
				set mpid_ [eval exec mb -sa $session_($sess,Address) &]
			}
	
		}
	}
}
